#!/usr/bin/env sh
set -eu
  
dry_run=${dry_run:-}

while [ $# -gt 0 ]; do
  case "$1" in
    --dry-run)
      dry_run=1
      shift
      ;;
    *)
      echo "Unknown argument: $1"
      exit 1
      ;;
  esac
done

is_dry_run() {
	if [ -z "$dry_run" ]; then
		return 1
	else
		return 0
	fi
}

get_distribution() {
  os_id=""
  # Every supported distro should have /etc/os-release 
  if [ -f /etc/os-release ]; then
    os_id="$(. /etc/os-release && echo $ID)"
  # Fall back to /etc/lsb-release if /etc/os-release is not available
  elif [ -f /etc/lsb-release ]; then
    os_id="$(. /etc/lsb-release && echo $DISTRIB_ID)"
  fi
  echo "$os_id"
}

get_version() {
  os_version=""
  if [ -f /etc/os-release ]; then
    os_version="$(. /etc/os-release && echo $VERSION_ID)"
  elif [ -f /etc/lsb-release ]; then
    os_version="$(. /etc/lsb-release && echo $DISTRIB_RELEASE)"
  fi
  echo "$os_version"
}

command_exists() {
  command -v "$1" >/dev/null 2>&1
}

main() {
  # Checking if xTend is already installed
  if  command_exists xTend; then
    echo "xTend has been installed. Run with 'xTend-backend'"
    exit 1
  fi

  user="$(id -un 2>/dev/null || true)"
  sh_c='sh -c'
  if [ "$user" != 'root' ]; then
    if command_exists sudo; then
      sh_c='sudo -E sh -c'
    elif command_exists su; then
      sh_c='su -c'
    else
			cat >&2 <<-'EOF'
			Error: this installer needs the ability to run commands as root.
			Sudo or su is required.
			EOF
			exit 1
    fi
  fi
	if is_dry_run; then
		sh_c='echo'
	fi

	os_id="$(get_distribution)"
	os_version="$(get_version)"

	case "$os_id" in
		ubuntu)
			case "$os_version" in
				24.04)
					$sh_c "apt install -y ./xtend*_amd64.deb"
				;;
				22.04)
					#TODO: Improve this snippet
					$sh_c "wget http://mirrors.edge.kernel.org/ubuntu/pool/main/libb/libbpf/libbpf1_1.1.0-1_amd64.deb"
					$sh_c "dpkg -i libbpf1_1.1.0-1_amd64.deb"
					$sh_c "apt install -y software-properties-common"
					$sh_c "add-apt-repository -y ppa:ubuntu-toolchain-r/test"
					$sh_c "apt update"
          $sh_c "apt install -y ./xtend*_amd64.deb"
					$sh_c "add-apt-repository -y --remove ppa:ubuntu-toolchain-r/test"
					$sh_c "apt update"
					$sh_c "rm libbpf1_1.1.0-1_amd64.deb"
					;;
				20.04)
				#TODO: Add support for Ubuntu 20.04
				;;
				*)
					echo "Unsupported OS version"
					exit 1
					;;
			esac
			;;
		debian)
			case "$os_version" in
				12)
					$sh_c "echo 'deb http://deb.debian.org/debian trixie main' | tee /etc/apt/sources.list.d/trixie.list"
					$sh_c "apt update"
					$sh_c "apt install -y ./xtend*_amd64.deb"
					$sh_c "rm -f /etc/apt/sources.list.d/trixie.list"
          $sh_c "apt update"
					;;
				11)
					#TODO: Add support for Debian 11
					echo "Unsupported OS version"
					exit 1
					;;
				*)
					echo "Unsupported OS version"
					exit 1
					;;
			esac
			;;
		#TODO: Add detection of forked distros as support increases
		*)
			echo "Unsupported OS"
			exit 1
			;;
	esac
}

main "$@"